@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: David Zeuthen <davidz@redhat.com>
  */
@@ -31,21 +33,20 @@
 #include "gdbusutils.h"
 #include "gioenumtypes.h"
 #include "gcredentials.h"
-#include "gcredentialsprivate.h"
 #include "gdbusprivate.h"
 #include "giostream.h"
 #include "gdatainputstream.h"
 #include "gdataoutputstream.h"
 
 #ifdef G_OS_UNIX
-#include "gnetworking.h"
+#include <sys/types.h>
+#include <sys/socket.h>
 #include "gunixconnection.h"
 #include "gunixcredentialsmessage.h"
 #endif
 
 #include "glibintl.h"
 
-G_GNUC_PRINTF(1, 2)
 static void
 debug_print (const gchar *message, ...)
 {
@@ -103,7 +104,7 @@ enum
   PROP_STREAM
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GDBusAuth, _g_dbus_auth, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GDBusAuth, _g_dbus_auth, G_TYPE_OBJECT);
 
 /* ---------------------------------------------------------------------------------------------------- */
 
@@ -114,7 +115,8 @@ _g_dbus_auth_finalize (GObject *object)
 
   if (auth->priv->stream != NULL)
     g_object_unref (auth->priv->stream);
-  g_list_free_full (auth->priv->available_mechanisms, (GDestroyNotify) mechanism_free);
+  g_list_foreach (auth->priv->available_mechanisms, (GFunc) mechanism_free, NULL);
+  g_list_free (auth->priv->available_mechanisms);
 
   if (G_OBJECT_CLASS (_g_dbus_auth_parent_class)->finalize != NULL)
     G_OBJECT_CLASS (_g_dbus_auth_parent_class)->finalize (object);
@@ -165,6 +167,8 @@ _g_dbus_auth_class_init (GDBusAuthClass *klass)
 {
   GObjectClass *gobject_class;
 
+  g_type_class_add_private (klass, sizeof (GDBusAuthPrivate));
+
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->get_property = _g_dbus_auth_get_property;
   gobject_class->set_property = _g_dbus_auth_set_property;
@@ -191,22 +195,17 @@ mechanism_free (Mechanism *m)
 }
 
 static void
-add_mechanism (GDBusAuth         *auth,
-               GDBusAuthObserver *observer,
-               GType              mechanism_type)
+add_mechanism (GDBusAuth *auth,
+               GType      mechanism_type)
 {
-  const gchar *name;
+  Mechanism *m;
 
-  name = _g_dbus_auth_mechanism_get_name (mechanism_type);
-  if (observer == NULL || g_dbus_auth_observer_allow_mechanism (observer, name))
-    {
-      Mechanism *m;
-      m = g_new0 (Mechanism, 1);
-      m->name = name;
-      m->priority = _g_dbus_auth_mechanism_get_priority (mechanism_type);
-      m->gtype = mechanism_type;
-      auth->priv->available_mechanisms = g_list_prepend (auth->priv->available_mechanisms, m);
-    }
+  m = g_new0 (Mechanism, 1);
+  m->name = _g_dbus_auth_mechanism_get_name (mechanism_type);
+  m->priority = _g_dbus_auth_mechanism_get_priority (mechanism_type);
+  m->gtype = mechanism_type;
+
+  auth->priv->available_mechanisms = g_list_prepend (auth->priv->available_mechanisms, m);
 }
 
 static gint
@@ -223,17 +222,12 @@ mech_compare_func (Mechanism *a, Mechanism *b)
 static void
 _g_dbus_auth_init (GDBusAuth *auth)
 {
-  auth->priv = _g_dbus_auth_get_instance_private (auth);
-}
+  auth->priv = G_TYPE_INSTANCE_GET_PRIVATE (auth, G_TYPE_DBUS_AUTH, GDBusAuthPrivate);
 
-static void
-_g_dbus_auth_add_mechs (GDBusAuth         *auth,
-                        GDBusAuthObserver *observer)
-{
   /* TODO: trawl extension points */
-  add_mechanism (auth, observer, G_TYPE_DBUS_AUTH_MECHANISM_ANON);
-  add_mechanism (auth, observer, G_TYPE_DBUS_AUTH_MECHANISM_SHA1);
-  add_mechanism (auth, observer, G_TYPE_DBUS_AUTH_MECHANISM_EXTERNAL);
+  add_mechanism (auth, G_TYPE_DBUS_AUTH_MECHANISM_ANON);
+  add_mechanism (auth, G_TYPE_DBUS_AUTH_MECHANISM_SHA1);
+  add_mechanism (auth, G_TYPE_DBUS_AUTH_MECHANISM_EXTERNAL);
 
   auth->priv->available_mechanisms = g_list_sort (auth->priv->available_mechanisms,
                                                   (GCompareFunc) mech_compare_func);
@@ -366,6 +360,12 @@ _my_g_input_stream_read_line_safe (GInputStream  *i,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void
+append_nibble (GString *s, gint val)
+{
+  g_string_append_c (s, val >= 10 ? ('a' + val - 10) : ('0' + val));
+}
+
 static gchar *
 hexdecode (const gchar  *str,
            gsize        *out_len,
@@ -391,7 +391,7 @@ hexdecode (const gchar  *str,
           g_set_error (error,
                        G_IO_ERROR,
                        G_IO_ERROR_FAILED,
-                       "Error hexdecoding string '%s' around position %d",
+                       "Error hexdecoding string `%s' around position %d",
                        str, n);
           goto out;
         }
@@ -399,17 +399,38 @@ hexdecode (const gchar  *str,
       g_string_append_c (s, value);
     }
 
-  *out_len = s->len;
   ret = g_string_free (s, FALSE);
   s = NULL;
 
  out:
   if (s != NULL)
+    g_string_free (s, TRUE);
+  return ret;
+}
+
+/* TODO: take len */
+static gchar *
+hexencode (const gchar *str)
+{
+  guint n;
+  GString *s;
+
+  s = g_string_new (NULL);
+  for (n = 0; str[n] != '\0'; n++)
     {
-      *out_len = 0;
-      g_string_free (s, TRUE);
+      gint val;
+      gint upper_nibble;
+      gint lower_nibble;
+
+      val = ((const guchar *) str)[n];
+      upper_nibble = val >> 4;
+      lower_nibble = val & 0x0f;
+
+      append_nibble (s, upper_nibble);
+      append_nibble (s, lower_nibble);
     }
-   return ret;
+
+  return g_string_free (s, FALSE);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -492,7 +513,7 @@ client_choose_mech_and_send_initial_response (GDBusAuth           *auth,
                        "stream", auth->priv->stream,
                        "credentials", credentials_that_were_sent,
                        NULL);
-  debug_print ("CLIENT: Trying mechanism '%s'", _g_dbus_auth_mechanism_get_name (auth_mech_to_use_gtype));
+  debug_print ("CLIENT: Trying mechanism `%s'", _g_dbus_auth_mechanism_get_name (auth_mech_to_use_gtype));
   g_ptr_array_add (attempted_auth_mechs, (gpointer) _g_dbus_auth_mechanism_get_name (auth_mech_to_use_gtype));
 
   /* the auth mechanism may not be supported
@@ -500,25 +521,25 @@ client_choose_mech_and_send_initial_response (GDBusAuth           *auth,
    */
   if (!_g_dbus_auth_mechanism_is_supported (mech))
     {
-      debug_print ("CLIENT: Mechanism '%s' says it is not supported", _g_dbus_auth_mechanism_get_name (auth_mech_to_use_gtype));
+      debug_print ("CLIENT: Mechanism `%s' says it is not supported", _g_dbus_auth_mechanism_get_name (auth_mech_to_use_gtype));
       g_object_unref (mech);
       mech = NULL;
       goto again;
     }
 
-  initial_response_len = 0;
+  initial_response_len = -1;
   initial_response = _g_dbus_auth_mechanism_client_initiate (mech,
                                                              &initial_response_len);
 #if 0
-  g_printerr ("using auth mechanism with name '%s' of type '%s' with initial response '%s'\n",
+  g_printerr ("using auth mechanism with name `%s' of type `%s' with initial response `%s'\n",
               _g_dbus_auth_mechanism_get_name (auth_mech_to_use_gtype),
               g_type_name (G_TYPE_FROM_INSTANCE (mech)),
               initial_response);
 #endif
   if (initial_response != NULL)
     {
-      //g_printerr ("initial_response = '%s'\n", initial_response);
-      encoded = _g_dbus_hexencode (initial_response, initial_response_len);
+      //g_printerr ("initial_response = `%s'\n", initial_response);
+      encoded = hexencode (initial_response);
       s = g_strdup_printf ("AUTH %s %s\r\n",
                            _g_dbus_auth_mechanism_get_name (auth_mech_to_use_gtype),
                            encoded);
@@ -529,7 +550,7 @@ client_choose_mech_and_send_initial_response (GDBusAuth           *auth,
     {
       s = g_strdup_printf ("AUTH %s\r\n", _g_dbus_auth_mechanism_get_name (auth_mech_to_use_gtype));
     }
-  debug_print ("CLIENT: writing '%s'", s);
+  debug_print ("CLIENT: writing `%s'", s);
   if (!g_data_output_stream_put_string (dos, s, cancellable, error))
     {
       g_object_unref (mech);
@@ -556,7 +577,6 @@ typedef enum
 
 gchar *
 _g_dbus_auth_run_client (GDBusAuth     *auth,
-                         GDBusAuthObserver     *observer,
                          GDBusCapabilityFlags offered_capabilities,
                          GDBusCapabilityFlags *out_negotiated_capabilities,
                          GCancellable  *cancellable,
@@ -577,8 +597,6 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
 
   debug_print ("CLIENT: initiating");
 
-  _g_dbus_auth_add_mechs (auth, observer);
-
   ret_guid = NULL;
   supported_auth_mechs = NULL;
   attempted_auth_mechs = g_ptr_array_new ();
@@ -594,7 +612,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
   g_data_input_stream_set_newline_type (dis, G_DATA_STREAM_NEWLINE_TYPE_CR_LF);
 
 #ifdef G_OS_UNIX
-  if (G_IS_UNIX_CONNECTION (auth->priv->stream))
+  if (G_IS_UNIX_CONNECTION (auth->priv->stream) && g_unix_credentials_message_is_supported ())
     {
       credentials = g_credentials_new ();
       if (!g_unix_connection_send_credentials (G_UNIX_CONNECTION (auth->priv->stream),
@@ -617,7 +635,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
       if (G_UNLIKELY (_g_dbus_debug_authentication ()))
         {
           s = g_credentials_to_string (credentials);
-          debug_print ("CLIENT: sent credentials '%s'", s);
+          debug_print ("CLIENT: sent credentials `%s'", s);
           g_free (s);
         }
     }
@@ -630,7 +648,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
 
   /* Get list of supported authentication mechanisms */
   s = "AUTH\r\n";
-  debug_print ("CLIENT: writing '%s'", s);
+  debug_print ("CLIENT: writing `%s'", s);
   if (!g_data_output_stream_put_string (dos, s, cancellable, error))
     goto out;
   state = CLIENT_STATE_WAITING_FOR_REJECT;
@@ -652,7 +670,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
               g_set_error (error,
                            G_IO_ERROR,
                            G_IO_ERROR_FAILED,
-                           "In WaitingForReject: Expected 'REJECTED am1 am2 ... amN', got '%s'",
+                           "In WaitingForReject: Expected `REJECTED am1 am2 ... amN', got `%s'",
                            line);
               g_free (line);
               goto out;
@@ -662,7 +680,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
               supported_auth_mechs = g_strsplit (line + sizeof ("REJECTED ") - 1, " ", 0);
 #if 0
               for (n = 0; supported_auth_mechs != NULL && supported_auth_mechs[n] != NULL; n++)
-                g_printerr ("supported_auth_mechs[%d] = '%s'\n", n, supported_auth_mechs[n]);
+                g_printerr ("supported_auth_mechs[%d] = `%s'\n", n, supported_auth_mechs[n]);
 #endif
             }
           g_free (line);
@@ -686,7 +704,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
           line = _my_g_data_input_stream_read_line (dis, &line_length, cancellable, error);
           if (line == NULL)
             goto out;
-          debug_print ("CLIENT: WaitingForOK, read '%s'", line);
+          debug_print ("CLIENT: WaitingForOK, read `%s'", line);
           if (g_str_has_prefix (line, "OK "))
             {
               if (!g_dbus_is_guid (line + 3))
@@ -694,7 +712,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
                   g_set_error (error,
                                G_IO_ERROR,
                                G_IO_ERROR_FAILED,
-                               "Invalid OK response '%s'",
+                               "Invalid OK response `%s'",
                                line);
                   g_free (line);
                   goto out;
@@ -705,7 +723,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
               if (offered_capabilities & G_DBUS_CAPABILITY_FLAGS_UNIX_FD_PASSING)
                 {
                   s = "NEGOTIATE_UNIX_FD\r\n";
-                  debug_print ("CLIENT: writing '%s'", s);
+                  debug_print ("CLIENT: writing `%s'", s);
                   if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                     goto out;
                   state = CLIENT_STATE_WAITING_FOR_AGREE_UNIX_FD;
@@ -713,7 +731,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
               else
                 {
                   s = "BEGIN\r\n";
-                  debug_print ("CLIENT: writing '%s'", s);
+                  debug_print ("CLIENT: writing `%s'", s);
                   if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                     goto out;
                   /* and we're done! */
@@ -730,7 +748,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
               g_set_error (error,
                            G_IO_ERROR,
                            G_IO_ERROR_FAILED,
-                           "In WaitingForOk: unexpected response '%s'",
+                           "In WaitingForOk: unexpected response `%s'",
                            line);
               g_free (line);
               goto out;
@@ -742,13 +760,13 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
           line = _my_g_data_input_stream_read_line (dis, &line_length, cancellable, error);
           if (line == NULL)
             goto out;
-          debug_print ("CLIENT: WaitingForAgreeUnixFD, read='%s'", line);
+          debug_print ("CLIENT: WaitingForAgreeUnixFD, read=`%s'", line);
           if (g_strcmp0 (line, "AGREE_UNIX_FD") == 0)
             {
               g_free (line);
               negotiated_capabilities |= G_DBUS_CAPABILITY_FLAGS_UNIX_FD_PASSING;
               s = "BEGIN\r\n";
-              debug_print ("CLIENT: writing '%s'", s);
+              debug_print ("CLIENT: writing `%s'", s);
               if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                 goto out;
               /* and we're done! */
@@ -756,10 +774,10 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
             }
           else if (g_str_has_prefix (line, "ERROR") && (line[5] == 0 || g_ascii_isspace (line[5])))
             {
-              //g_strstrip (line + 5); g_debug ("bah, no unix_fd: '%s'", line + 5);
+              //g_strstrip (line + 5); g_debug ("bah, no unix_fd: `%s'", line + 5);
               g_free (line);
               s = "BEGIN\r\n";
-              debug_print ("CLIENT: writing '%s'", s);
+              debug_print ("CLIENT: writing `%s'", s);
               if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                 goto out;
               /* and we're done! */
@@ -771,7 +789,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
               g_set_error (error,
                            G_IO_ERROR,
                            G_IO_ERROR_FAILED,
-                           "In WaitingForAgreeUnixFd: unexpected response '%s'",
+                           "In WaitingForAgreeUnixFd: unexpected response `%s'",
                            line);
               g_free (line);
               goto out;
@@ -783,14 +801,14 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
           line = _my_g_data_input_stream_read_line (dis, &line_length, cancellable, error);
           if (line == NULL)
             goto out;
-          debug_print ("CLIENT: WaitingForData, read='%s'", line);
-          if (g_str_equal (line, "DATA") || g_str_has_prefix (line, "DATA "))
+          debug_print ("CLIENT: WaitingForData, read=`%s'", line);
+          if (g_str_has_prefix (line, "DATA "))
             {
               gchar *encoded;
               gchar *decoded_data;
               gsize decoded_data_len = 0;
 
-              encoded = g_strdup (line + 4);
+              encoded = g_strdup (line + 5);
               g_free (line);
               g_strstrip (encoded);
               decoded_data = hexdecode (encoded, &decoded_data_len, error);
@@ -810,11 +828,11 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
                   gsize data_len;
                   gchar *encoded_data;
                   data = _g_dbus_auth_mechanism_client_data_send (mech, &data_len);
-                  encoded_data = _g_dbus_hexencode (data, data_len);
+                  encoded_data = hexencode (data);
                   s = g_strdup_printf ("DATA %s\r\n", encoded_data);
                   g_free (encoded_data);
                   g_free (data);
-                  debug_print ("CLIENT: writing '%s'", s);
+                  debug_print ("CLIENT: writing `%s'", s);
                   if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                     {
                       g_free (s);
@@ -836,7 +854,7 @@ _g_dbus_auth_run_client (GDBusAuth     *auth,
               g_set_error (error,
                            G_IO_ERROR,
                            G_IO_ERROR_FAILED,
-                           "In WaitingForData: unexpected response '%s'",
+                           "In WaitingForData: unexpected response `%s'",
                            line);
               g_free (line);
               goto out;
@@ -924,7 +942,6 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                          GDBusAuthObserver      *observer,
                          const gchar            *guid,
                          gboolean                allow_anonymous,
-                         gboolean                require_same_user,
                          GDBusCapabilityFlags    offered_capabilities,
                          GDBusCapabilityFlags   *out_negotiated_capabilities,
                          GCredentials          **out_received_credentials,
@@ -936,18 +953,16 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
   GDataInputStream *dis;
   GDataOutputStream *dos;
   GError *local_error;
+  guchar byte;
   gchar *line;
   gsize line_length;
   GDBusAuthMechanism *mech;
   gchar *s;
   GDBusCapabilityFlags negotiated_capabilities;
   GCredentials *credentials;
-  GCredentials *own_credentials = NULL;
 
   debug_print ("SERVER: initiating");
 
-  _g_dbus_auth_add_mechs (auth, observer);
-
   ret = FALSE;
   dis = NULL;
   dos = NULL;
@@ -960,7 +975,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
       g_set_error (error,
                    G_IO_ERROR,
                    G_IO_ERROR_FAILED,
-                   "The given GUID '%s' is not valid",
+                   "The given guid `%s' is not valid",
                    guid);
       goto out;
     }
@@ -972,47 +987,24 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
 
   g_data_input_stream_set_newline_type (dis, G_DATA_STREAM_NEWLINE_TYPE_CR_LF);
 
-  /* read the NUL-byte, possibly with credentials attached */
+  /* first read the NUL-byte (TODO: read credentials if using a unix domain socket) */
 #ifdef G_OS_UNIX
-#ifndef G_CREDENTIALS_PREFER_MESSAGE_PASSING
-  if (G_IS_SOCKET_CONNECTION (auth->priv->stream))
-    {
-      GSocket *sock = g_socket_connection_get_socket (G_SOCKET_CONNECTION (auth->priv->stream));
-
-      local_error = NULL;
-      credentials = g_socket_get_credentials (sock, &local_error);
-
-      if (credentials == NULL && !g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED))
-        {
-          g_propagate_error (error, local_error);
-          goto out;
-        }
-      else
-        {
-          /* Clear the error indicator, so we can retry with
-           * g_unix_connection_receive_credentials() if necessary */
-          g_clear_error (&local_error);
-        }
-    }
-#endif
-
-  if (credentials == NULL && G_IS_UNIX_CONNECTION (auth->priv->stream))
+  if (G_IS_UNIX_CONNECTION (auth->priv->stream) && g_unix_credentials_message_is_supported ())
     {
       local_error = NULL;
       credentials = g_unix_connection_receive_credentials (G_UNIX_CONNECTION (auth->priv->stream),
                                                            cancellable,
                                                            &local_error);
-      if (credentials == NULL && !g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED))
+      if (credentials == NULL)
         {
           g_propagate_error (error, local_error);
           goto out;
         }
-      g_clear_error (&local_error);
     }
   else
     {
       local_error = NULL;
-      (void)g_data_input_stream_read_byte (dis, cancellable, &local_error);
+      byte = g_data_input_stream_read_byte (dis, cancellable, &local_error);
       if (local_error != NULL)
         {
           g_propagate_error (error, local_error);
@@ -1021,7 +1013,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
     }
 #else
   local_error = NULL;
-  (void)g_data_input_stream_read_byte (dis, cancellable, &local_error);
+  byte = g_data_input_stream_read_byte (dis, cancellable, &local_error);
   if (local_error != NULL)
     {
       g_propagate_error (error, local_error);
@@ -1033,7 +1025,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
       if (G_UNLIKELY (_g_dbus_debug_authentication ()))
         {
           s = g_credentials_to_string (credentials);
-          debug_print ("SERVER: received credentials '%s'", s);
+          debug_print ("SERVER: received credentials `%s'", s);
           g_free (s);
         }
     }
@@ -1042,8 +1034,6 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
       debug_print ("SERVER: didn't receive any credentials");
     }
 
-  own_credentials = g_credentials_new ();
-
   state = SERVER_STATE_WAITING_FOR_AUTH;
   while (TRUE)
     {
@@ -1052,17 +1042,16 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
         case SERVER_STATE_WAITING_FOR_AUTH:
           debug_print ("SERVER: WaitingForAuth");
           line = _my_g_data_input_stream_read_line (dis, &line_length, cancellable, error);
-          debug_print ("SERVER: WaitingForAuth, read '%s'", line);
+          debug_print ("SERVER: WaitingForAuth, read `%s'", line);
           if (line == NULL)
             goto out;
           if (g_strcmp0 (line, "AUTH") == 0)
             {
               s = get_auth_mechanisms (auth, allow_anonymous, "REJECTED ", "\r\n", " ");
-              debug_print ("SERVER: writing '%s'", s);
+              debug_print ("SERVER: writing `%s'", s);
               if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                 {
                   g_free (s);
-                  g_free (line);
                   goto out;
                 }
               g_free (s);
@@ -1076,6 +1065,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
               GType auth_mech_to_use_gtype;
 
               tokens = g_strsplit (line, " ", 0);
+              g_free (line);
 
               switch (g_strv_length (tokens))
                 {
@@ -1095,17 +1085,14 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                   g_set_error (error,
                                G_IO_ERROR,
                                G_IO_ERROR_FAILED,
-                               "Unexpected line '%s' while in WaitingForAuth state",
+                               "Unexpected line `%s' while in WaitingForAuth state",
                                line);
                   g_strfreev (tokens);
-                  g_free (line);
                   goto out;
                 }
 
-              g_free (line);
-
               /* TODO: record that the client has attempted to use this mechanism */
-              //g_debug ("client is trying '%s'", mech_name);
+              //g_debug ("client is trying `%s'", mech_name);
 
               auth_mech_to_use_gtype = find_mech_by_name (auth, mech_name);
               if ((auth_mech_to_use_gtype == (GType) 0) ||
@@ -1114,7 +1101,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                   /* We don't support this auth mechanism */
                   g_strfreev (tokens);
                   s = get_auth_mechanisms (auth, allow_anonymous, "REJECTED ", "\r\n", " ");
-                  debug_print ("SERVER: writing '%s'", s);
+                  debug_print ("SERVER: writing `%s'", s);
                   if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                     {
                       g_free (s);
@@ -1130,7 +1117,6 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                   gchar *initial_response;
                   gsize initial_response_len;
 
-                  g_clear_object (&mech);
                   mech = g_object_new (auth_mech_to_use_gtype,
                                        "stream", auth->priv->stream,
                                        "credentials", credentials,
@@ -1160,21 +1146,10 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                   switch (_g_dbus_auth_mechanism_server_get_state (mech))
                     {
                     case G_DBUS_AUTH_MECHANISM_STATE_ACCEPTED:
-                      if (require_same_user &&
-                          (credentials == NULL ||
-                           !g_credentials_is_same_user (credentials, own_credentials, NULL)))
-                        {
-                          /* disconnect */
-                          g_set_error_literal (error,
-                                               G_IO_ERROR,
-                                               G_IO_ERROR_FAILED,
-                                               _("User IDs must be the same for peer and server"));
-                          goto out;
-                        }
-                      else if (observer != NULL &&
-                               !g_dbus_auth_observer_authorize_authenticated_peer (observer,
-                                                                                   auth->priv->stream,
-                                                                                   credentials))
+                      if (observer != NULL &&
+                          !g_dbus_auth_observer_authorize_authenticated_peer (observer,
+                                                                              auth->priv->stream,
+                                                                              credentials))
                         {
                           /* disconnect */
                           g_set_error_literal (error,
@@ -1186,7 +1161,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                       else
                         {
                           s = g_strdup_printf ("OK %s\r\n", guid);
-                          debug_print ("SERVER: writing '%s'", s);
+                          debug_print ("SERVER: writing `%s'", s);
                           if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                             {
                               g_free (s);
@@ -1199,7 +1174,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
 
                     case G_DBUS_AUTH_MECHANISM_STATE_REJECTED:
                       s = get_auth_mechanisms (auth, allow_anonymous, "REJECTED ", "\r\n", " ");
-                      debug_print ("SERVER: writing '%s'", s);
+                      debug_print ("SERVER: writing `%s'", s);
                       if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                         {
                           g_free (s);
@@ -1217,25 +1192,19 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                       {
                         gchar *data;
                         gsize data_len;
-
+                        gchar *encoded_data;
                         data = _g_dbus_auth_mechanism_server_data_send (mech, &data_len);
-                        if (data != NULL)
+                        encoded_data = hexencode (data);
+                        s = g_strdup_printf ("DATA %s\r\n", encoded_data);
+                        g_free (encoded_data);
+                        g_free (data);
+                        debug_print ("SERVER: writing `%s'", s);
+                        if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                           {
-                            gchar *encoded_data;
-
-                            encoded_data = _g_dbus_hexencode (data, data_len);
-                            s = g_strdup_printf ("DATA %s\r\n", encoded_data);
-                            g_free (encoded_data);
-                            g_free (data);
-
-                            debug_print ("SERVER: writing '%s'", s);
-                            if (!g_data_output_stream_put_string (dos, s, cancellable, error))
-                              {
-                                g_free (s);
-                                goto out;
-                              }
                             g_free (s);
+                            goto out;
                           }
+                        g_free (s);
                       }
                       goto change_state;
                       break;
@@ -1252,7 +1221,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
               g_set_error (error,
                            G_IO_ERROR,
                            G_IO_ERROR_FAILED,
-                           "Unexpected line '%s' while in WaitingForAuth state",
+                           "Unexpected line `%s' while in WaitingForAuth state",
                            line);
               g_free (line);
               goto out;
@@ -1262,16 +1231,16 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
         case SERVER_STATE_WAITING_FOR_DATA:
           debug_print ("SERVER: WaitingForData");
           line = _my_g_data_input_stream_read_line (dis, &line_length, cancellable, error);
-          debug_print ("SERVER: WaitingForData, read '%s'", line);
+          debug_print ("SERVER: WaitingForData, read `%s'", line);
           if (line == NULL)
             goto out;
-          if (g_str_equal (line, "DATA") || g_str_has_prefix (line, "DATA "))
+          if (g_str_has_prefix (line, "DATA "))
             {
               gchar *encoded;
               gchar *decoded_data;
               gsize decoded_data_len = 0;
 
-              encoded = g_strdup (line + 4);
+              encoded = g_strdup (line + 5);
               g_free (line);
               g_strstrip (encoded);
               decoded_data = hexdecode (encoded, &decoded_data_len, error);
@@ -1292,7 +1261,7 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
               g_set_error (error,
                            G_IO_ERROR,
                            G_IO_ERROR_FAILED,
-                           "Unexpected line '%s' while in WaitingForData state",
+                           "Unexpected line `%s' while in WaitingForData state",
                            line);
               g_free (line);
             }
@@ -1311,9 +1280,9 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                                                     &line_length,
                                                     cancellable,
                                                     error);
+          debug_print ("SERVER: WaitingForBegin, read `%s'", line);
           if (line == NULL)
             goto out;
-          debug_print ("SERVER: WaitingForBegin, read '%s'", line);
           if (g_strcmp0 (line, "BEGIN") == 0)
             {
               /* YAY, done! */
@@ -1328,24 +1297,24 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                 {
                   negotiated_capabilities |= G_DBUS_CAPABILITY_FLAGS_UNIX_FD_PASSING;
                   s = "AGREE_UNIX_FD\r\n";
-                  debug_print ("SERVER: writing '%s'", s);
+                  debug_print ("SERVER: writing `%s'", s);
                   if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                     goto out;
                 }
               else
                 {
                   s = "ERROR \"fd passing not offered\"\r\n";
-                  debug_print ("SERVER: writing '%s'", s);
+                  debug_print ("SERVER: writing `%s'", s);
                   if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                     goto out;
                 }
             }
           else
             {
-              g_debug ("Unexpected line '%s' while in WaitingForBegin state", line);
+              g_debug ("Unexpected line `%s' while in WaitingForBegin state", line);
               g_free (line);
               s = "ERROR \"Unknown Command\"\r\n";
-              debug_print ("SERVER: writing '%s'", s);
+              debug_print ("SERVER: writing `%s'", s);
               if (!g_data_output_stream_put_string (dos, s, cancellable, error))
                 goto out;
             }
@@ -1364,10 +1333,12 @@ _g_dbus_auth_run_server (GDBusAuth              *auth,
                        "Not implemented (server)");
 
  out:
-  g_clear_object (&mech);
-  g_clear_object (&dis);
-  g_clear_object (&dos);
-  g_clear_object (&own_credentials);
+  if (mech != NULL)
+    g_object_unref (mech);
+  if (dis != NULL)
+    g_object_unref (dis);
+  if (dos != NULL)
+    g_object_unref (dos);
 
   /* ensure return value is FALSE if error is set */
   if (error != NULL && *error != NULL)
