@@ -4,12 +4,10 @@
  * 
  * Copyright (C) 2006-2008 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
  *         David Zeuthen <davidz@redhat.com>
@@ -29,41 +29,37 @@
 
 #include "gmount.h"
 #include "gmountprivate.h"
-#include "gthemedicon.h"
 #include "gasyncresult.h"
-#include "gtask.h"
+#include "gsimpleasyncresult.h"
 #include "gioerror.h"
 #include "glibintl.h"
 
 
 /**
- * GMount:
+ * SECTION:gmount
+ * @short_description: Mount management
+ * @include: gio/gio.h
+ * @see_also: GVolume, GUnixMount
  *
- * The `GMount` interface represents a user-visible mount, such as a mounted
- * file system.
+ * The #GMount interface represents user-visible mounts. Note, when 
+ * porting from GnomeVFS, #GMount is the moral equivalent of #GnomeVFSVolume.
  *
- * `GMount` is a ‘mounted’ filesystem that you can access. Mounted is in
- * quotes because it’s not the same as a UNIX mount, it might be a GVFS
- * mount, but you can still access the files on it if you use GIO.
- *
- * A `GMount` might be associated with a [iface@Gio.Volume] (such as a USB flash
- * drive) which hosts it.
+ * #GMount is a "mounted" filesystem that you can access. Mounted is in
+ * quotes because it's not the same as a unix mount, it might be a gvfs
+ * mount, but you can still access the files on it if you use GIO. Might or
+ * might not be related to a volume object.
  * 
- * Unmounting a `GMount` instance is an asynchronous operation. For
- * more information about asynchronous operations, see [iface@Gio.AsyncResult]
- * and [class@Gio.Task]. To unmount a `GMount` instance, first call
- * [method@Gio.Mount.unmount_with_operation] with (at least) the `GMount`
- * instance and a [type@Gio.AsyncReadyCallback].  The callback will be fired
- * when the operation has resolved (either with success or failure), and a
- * [iface@Gio.AsyncResult] structure will be passed to the callback.  That
- * callback should then call [method@Gio.Mount.unmount_with_operation_finish]
- * with the `GMount` and the [iface@Gio.AsyncResult] data to see if the
- * operation was completed successfully.  If an `error` is present when
- * [method@Gio.Mount.unmount_with_operation_finish] is called, then it will be
- * filled with any error information.
- *
- * Note, when [porting from GnomeVFS](migrating-gnome-vfs.html), `GMount` is the
- * moral equivalent of `GnomeVFSVolume`.
+ * Unmounting a #GMount instance is an asynchronous operation. For
+ * more information about asynchronous operations, see #GAsyncReady
+ * and #GSimpleAsyncReady. To unmount a #GMount instance, first call
+ * g_mount_unmount_with_operation() with (at least) the #GMount instance and a
+ * #GAsyncReadyCallback.  The callback will be fired when the
+ * operation has resolved (either with success or failure), and a
+ * #GAsyncReady structure will be passed to the callback.  That
+ * callback should then call g_mount_unmount_with_operation_finish() with the #GMount
+ * and the #GAsyncReady data to see if the operation was completed
+ * successfully.  If an @error is present when g_mount_unmount_with_operation_finish() 
+ * is called, then it will be filled with any error information.
  **/
 
 typedef GMountIface GMountInterface;
@@ -83,7 +79,7 @@ g_mount_default_init (GMountInterface *iface)
                 G_SIGNAL_RUN_LAST,
                 G_STRUCT_OFFSET (GMountIface, changed),
                 NULL, NULL,
-                NULL,
+                g_cclosure_marshal_VOID__VOID,
                 G_TYPE_NONE, 0);
 
   /**
@@ -100,18 +96,15 @@ g_mount_default_init (GMountInterface *iface)
                 G_SIGNAL_RUN_LAST,
                 G_STRUCT_OFFSET (GMountIface, unmounted),
                 NULL, NULL,
-                NULL,
+                g_cclosure_marshal_VOID__VOID,
                 G_TYPE_NONE, 0);
   /**
    * GMount::pre-unmount:
    * @mount: the object on which the signal is emitted
    *
-   * This signal may be emitted when the #GMount is about to be
+   * This signal is emitted when the #GMount is about to be
    * unmounted.
    *
-   * This signal depends on the backend and is only emitted if
-   * GIO was used to unmount.
-   *
    * Since: 2.22
    **/
   g_signal_new (I_("pre-unmount"),
@@ -119,7 +112,7 @@ g_mount_default_init (GMountInterface *iface)
                 G_SIGNAL_RUN_LAST,
                 G_STRUCT_OFFSET (GMountIface, pre_unmount),
                 NULL, NULL,
-                NULL,
+                g_cclosure_marshal_VOID__VOID,
                 G_TYPE_NONE, 0);
 }
 
@@ -220,37 +213,6 @@ g_mount_get_icon (GMount *mount)
   return (* iface->get_icon) (mount);
 }
 
-
-/**
- * g_mount_get_symbolic_icon:
- * @mount: a #GMount.
- * 
- * Gets the symbolic icon for @mount.
- * 
- * Returns: (transfer full): a #GIcon.
- *      The returned object should be unreffed with 
- *      g_object_unref() when no longer needed.
- *
- * Since: 2.34
- **/
-GIcon *
-g_mount_get_symbolic_icon (GMount *mount)
-{
-  GMountIface *iface;
-  GIcon *ret;
-
-  g_return_val_if_fail (G_IS_MOUNT (mount), NULL);
-
-  iface = G_MOUNT_GET_IFACE (mount);
-
-  if (iface->get_symbolic_icon != NULL)
-    ret = iface->get_symbolic_icon (mount);
-  else
-    ret = g_themed_icon_new_with_default_fallbacks ("folder-remote-symbolic");
-
-  return ret;
-}
-
 /**
  * g_mount_get_uuid:
  * @mount: a #GMount.
@@ -260,8 +222,7 @@ g_mount_get_symbolic_icon (GMount *mount)
  * considered an opaque string. Returns %NULL if there is no UUID
  * available.
  * 
- * Returns: (nullable) (transfer full): the UUID for @mount or %NULL if no UUID
- *     can be computed.
+ * Returns: the UUID for @mount or %NULL if no UUID can be computed.
  *     The returned string should be freed with g_free()
  *     when no longer needed.
  **/
@@ -282,9 +243,8 @@ g_mount_get_uuid (GMount *mount)
  * @mount: a #GMount.
  * 
  * Gets the volume for the @mount.
- *
- * Returns: (transfer full) (nullable): a #GVolume or %NULL if @mount is not
- *      associated with a volume.
+ * 
+ * Returns: (transfer full): a #GVolume or %NULL if @mount is not associated with a volume.
  *      The returned object should be unreffed with 
  *      g_object_unref() when no longer needed.
  **/
@@ -309,8 +269,7 @@ g_mount_get_volume (GMount *mount)
  * This is a convenience method for getting the #GVolume and then
  * using that object to get the #GDrive.
  * 
- * Returns: (transfer full) (nullable): a #GDrive or %NULL if @mount is not
- *      associated with a volume or a drive.
+ * Returns: (transfer full): a #GDrive or %NULL if @mount is not associated with a volume or a drive.
  *      The returned object should be unreffed with 
  *      g_object_unref() when no longer needed.
  **/
@@ -330,7 +289,7 @@ g_mount_get_drive (GMount *mount)
  * g_mount_can_unmount: 
  * @mount: a #GMount.
  * 
- * Checks if @mount can be unmounted.
+ * Checks if @mount can be mounted.
  * 
  * Returns: %TRUE if the @mount can be unmounted.
  **/
@@ -350,7 +309,7 @@ g_mount_can_unmount (GMount *mount)
  * g_mount_can_eject: 
  * @mount: a #GMount.
  * 
- * Checks if @mount can be ejected.
+ * Checks if @mount can be eject.
  * 
  * Returns: %TRUE if the @mount can be ejected.
  **/
@@ -370,8 +329,8 @@ g_mount_can_eject (GMount *mount)
  * g_mount_unmount:
  * @mount: a #GMount.
  * @flags: flags affecting the operation
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
- * @callback: (nullable): a #GAsyncReadyCallback, or %NULL.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
+ * @callback: (allow-none): a #GAsyncReadyCallback, or %NULL.
  * @user_data: user data passed to @callback.
  * 
  * Unmounts a mount. This is an asynchronous operation, and is 
@@ -395,13 +354,14 @@ g_mount_unmount (GMount              *mount,
 
   if (iface->unmount == NULL)
     {
-      g_task_report_new_error (mount, callback, user_data,
-                               g_mount_unmount_with_operation,
-                               G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                               /* Translators: This is an error
-                                * message for mount objects that
-                                * don't implement unmount. */
-                               _("mount doesn’t implement “unmount”"));
+      g_simple_async_report_error_in_idle (G_OBJECT (mount),
+					   callback, user_data,
+					   G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+					   /* Translators: This is an error
+					    * message for mount objects that
+					    * don't implement unmount. */
+					   _("mount doesn't implement \"unmount\""));
+
       return;
     }
   
@@ -412,7 +372,7 @@ g_mount_unmount (GMount              *mount,
  * g_mount_unmount_finish:
  * @mount: a #GMount.
  * @result: a #GAsyncResult.
- * @error: a #GError location to store the error occurring, or %NULL to 
+ * @error: a #GError location to store the error occuring, or %NULL to 
  *     ignore.
  * 
  * Finishes unmounting a mount. If any errors occurred during the operation, 
@@ -432,10 +392,12 @@ g_mount_unmount_finish (GMount        *mount,
   g_return_val_if_fail (G_IS_MOUNT (mount), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
 
-  if (g_async_result_legacy_propagate_error (result, error))
-    return FALSE;
-  else if (g_async_result_is_tagged (result, g_mount_unmount_with_operation))
-    return g_task_propagate_boolean (G_TASK (result), error);
+  if (G_IS_SIMPLE_ASYNC_RESULT (result))
+    {
+      GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+      if (g_simple_async_result_propagate_error (simple, error))
+        return FALSE;
+    }
   
   iface = G_MOUNT_GET_IFACE (mount);
   return (* iface->unmount_finish) (mount, result, error);
@@ -446,8 +408,8 @@ g_mount_unmount_finish (GMount        *mount,
  * g_mount_eject:
  * @mount: a #GMount.
  * @flags: flags affecting the unmount if required for eject
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
- * @callback: (nullable): a #GAsyncReadyCallback, or %NULL.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
+ * @callback: (allow-none): a #GAsyncReadyCallback, or %NULL.
  * @user_data: user data passed to @callback.
  * 
  * Ejects a mount. This is an asynchronous operation, and is 
@@ -471,13 +433,14 @@ g_mount_eject (GMount              *mount,
 
   if (iface->eject == NULL)
     {
-      g_task_report_new_error (mount, callback, user_data,
-                               g_mount_eject_with_operation,
-                               G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                               /* Translators: This is an error
-                                * message for mount objects that
-                                * don't implement eject. */
-                               _("mount doesn’t implement “eject”"));
+      g_simple_async_report_error_in_idle (G_OBJECT (mount),
+					   callback, user_data,
+					   G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+					   /* Translators: This is an error
+					    * message for mount objects that
+					    * don't implement eject. */
+					   _("mount doesn't implement \"eject\""));
+      
       return;
     }
   
@@ -488,7 +451,7 @@ g_mount_eject (GMount              *mount,
  * g_mount_eject_finish:
  * @mount: a #GMount.
  * @result: a #GAsyncResult.
- * @error: a #GError location to store the error occurring, or %NULL to 
+ * @error: a #GError location to store the error occuring, or %NULL to 
  *     ignore.
  * 
  * Finishes ejecting a mount. If any errors occurred during the operation, 
@@ -508,10 +471,12 @@ g_mount_eject_finish (GMount        *mount,
   g_return_val_if_fail (G_IS_MOUNT (mount), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
 
-  if (g_async_result_legacy_propagate_error (result, error))
-    return FALSE;
-  else if (g_async_result_is_tagged (result, g_mount_eject_with_operation))
-    return g_task_propagate_boolean (G_TASK (result), error);
+  if (G_IS_SIMPLE_ASYNC_RESULT (result))
+    {
+      GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+      if (g_simple_async_result_propagate_error (simple, error))
+        return FALSE;
+    }
   
   iface = G_MOUNT_GET_IFACE (mount);
   return (* iface->eject_finish) (mount, result, error);
@@ -521,10 +486,10 @@ g_mount_eject_finish (GMount        *mount,
  * g_mount_unmount_with_operation:
  * @mount: a #GMount.
  * @flags: flags affecting the operation
- * @mount_operation: (nullable): a #GMountOperation or %NULL to avoid
+ * @mount_operation: (allow-none): a #GMountOperation or %NULL to avoid
  *     user interaction.
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
- * @callback: (nullable): a #GAsyncReadyCallback, or %NULL.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
+ * @callback: (allow-none): a #GAsyncReadyCallback, or %NULL.
  * @user_data: user data passed to @callback.
  *
  * Unmounts a mount. This is an asynchronous operation, and is
@@ -549,13 +514,14 @@ g_mount_unmount_with_operation (GMount              *mount,
 
   if (iface->unmount == NULL && iface->unmount_with_operation == NULL)
     {
-      g_task_report_new_error (mount, callback, user_data,
-                               g_mount_unmount_with_operation,
-                               G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                               /* Translators: This is an error
-                                * message for mount objects that
-                                * don't implement any of unmount or unmount_with_operation. */
-                               _("mount doesn’t implement “unmount” or “unmount_with_operation”"));
+      g_simple_async_report_error_in_idle (G_OBJECT (mount),
+					   callback, user_data,
+					   G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+					   /* Translators: This is an error
+					    * message for mount objects that
+					    * don't implement any of unmount or unmount_with_operation. */
+					   _("mount doesn't implement \"unmount\" or \"unmount_with_operation\""));
+
       return;
     }
 
@@ -569,7 +535,7 @@ g_mount_unmount_with_operation (GMount              *mount,
  * g_mount_unmount_with_operation_finish:
  * @mount: a #GMount.
  * @result: a #GAsyncResult.
- * @error: a #GError location to store the error occurring, or %NULL to
+ * @error: a #GError location to store the error occuring, or %NULL to
  *     ignore.
  *
  * Finishes unmounting a mount. If any errors occurred during the operation,
@@ -589,10 +555,12 @@ g_mount_unmount_with_operation_finish (GMount        *mount,
   g_return_val_if_fail (G_IS_MOUNT (mount), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
 
-  if (g_async_result_legacy_propagate_error (result, error))
-    return FALSE;
-  else if (g_async_result_is_tagged (result, g_mount_unmount_with_operation))
-    return g_task_propagate_boolean (G_TASK (result), error);
+  if (G_IS_SIMPLE_ASYNC_RESULT (result))
+    {
+      GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+      if (g_simple_async_result_propagate_error (simple, error))
+        return FALSE;
+    }
 
   iface = G_MOUNT_GET_IFACE (mount);
   if (iface->unmount_with_operation_finish != NULL)
@@ -606,10 +574,10 @@ g_mount_unmount_with_operation_finish (GMount        *mount,
  * g_mount_eject_with_operation:
  * @mount: a #GMount.
  * @flags: flags affecting the unmount if required for eject
- * @mount_operation: (nullable): a #GMountOperation or %NULL to avoid
+ * @mount_operation: (allow-none): a #GMountOperation or %NULL to avoid
  *     user interaction.
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
- * @callback: (nullable): a #GAsyncReadyCallback, or %NULL.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
+ * @callback: (allow-none): a #GAsyncReadyCallback, or %NULL.
  * @user_data: user data passed to @callback.
  *
  * Ejects a mount. This is an asynchronous operation, and is
@@ -634,13 +602,13 @@ g_mount_eject_with_operation (GMount              *mount,
 
   if (iface->eject == NULL && iface->eject_with_operation == NULL)
     {
-      g_task_report_new_error (mount, callback, user_data,
-                               g_mount_eject_with_operation,
-                               G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                               /* Translators: This is an error
-                                * message for mount objects that
-                                * don't implement any of eject or eject_with_operation. */
-                               _("mount doesn’t implement “eject” or “eject_with_operation”"));
+      g_simple_async_report_error_in_idle (G_OBJECT (mount),
+					   callback, user_data,
+					   G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+					   /* Translators: This is an error
+					    * message for mount objects that
+					    * don't implement any of eject or eject_with_operation. */
+					   _("mount doesn't implement \"eject\" or \"eject_with_operation\""));
       return;
     }
 
@@ -654,7 +622,7 @@ g_mount_eject_with_operation (GMount              *mount,
  * g_mount_eject_with_operation_finish:
  * @mount: a #GMount.
  * @result: a #GAsyncResult.
- * @error: a #GError location to store the error occurring, or %NULL to
+ * @error: a #GError location to store the error occuring, or %NULL to
  *     ignore.
  *
  * Finishes ejecting a mount. If any errors occurred during the operation,
@@ -674,10 +642,12 @@ g_mount_eject_with_operation_finish (GMount        *mount,
   g_return_val_if_fail (G_IS_MOUNT (mount), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
 
-  if (g_async_result_legacy_propagate_error (result, error))
-    return FALSE;
-  else if (g_async_result_is_tagged (result, g_mount_eject_with_operation))
-    return g_task_propagate_boolean (G_TASK (result), error);
+  if (G_IS_SIMPLE_ASYNC_RESULT (result))
+    {
+      GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+      if (g_simple_async_result_propagate_error (simple, error))
+        return FALSE;
+    }
 
   iface = G_MOUNT_GET_IFACE (mount);
   if (iface->eject_with_operation_finish != NULL)
@@ -690,10 +660,10 @@ g_mount_eject_with_operation_finish (GMount        *mount,
  * g_mount_remount:
  * @mount: a #GMount.
  * @flags: flags affecting the operation
- * @mount_operation: (nullable): a #GMountOperation or %NULL to avoid
+ * @mount_operation: (allow-none): a #GMountOperation or %NULL to avoid
  *     user interaction.
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
- * @callback: (nullable): a #GAsyncReadyCallback, or %NULL.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
+ * @callback: (allow-none): a #GAsyncReadyCallback, or %NULL.
  * @user_data: user data passed to @callback.
  * 
  * Remounts a mount. This is an asynchronous operation, and is 
@@ -722,13 +692,14 @@ g_mount_remount (GMount              *mount,
 
   if (iface->remount == NULL)
     { 
-      g_task_report_new_error (mount, callback, user_data,
-                               g_mount_remount,
-                               G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                               /* Translators: This is an error
-                                * message for mount objects that
-                                * don't implement remount. */
-                               _("mount doesn’t implement “remount”"));
+      g_simple_async_report_error_in_idle (G_OBJECT (mount),
+					   callback, user_data,
+					   G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+					   /* Translators: This is an error
+					    * message for mount objects that
+					    * don't implement remount. */
+					   _("mount doesn't implement \"remount\""));
+      
       return;
     }
   
@@ -739,7 +710,7 @@ g_mount_remount (GMount              *mount,
  * g_mount_remount_finish:
  * @mount: a #GMount.
  * @result: a #GAsyncResult.
- * @error: a #GError location to store the error occurring, or %NULL to 
+ * @error: a #GError location to store the error occuring, or %NULL to 
  *     ignore.
  * 
  * Finishes remounting a mount. If any errors occurred during the operation, 
@@ -757,10 +728,12 @@ g_mount_remount_finish (GMount        *mount,
   g_return_val_if_fail (G_IS_MOUNT (mount), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
 
-  if (g_async_result_legacy_propagate_error (result, error))
-    return FALSE;
-  else if (g_async_result_is_tagged (result, g_mount_remount))
-    return g_task_propagate_boolean (G_TASK (result), error);
+  if (G_IS_SIMPLE_ASYNC_RESULT (result))
+    {
+      GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+      if (g_simple_async_result_propagate_error (simple, error))
+        return FALSE;
+    }
   
   iface = G_MOUNT_GET_IFACE (mount);
   return (* iface->remount_finish) (mount, result, error);
@@ -771,15 +744,14 @@ g_mount_remount_finish (GMount        *mount,
  * @mount: a #GMount
  * @force_rescan: Whether to force a rescan of the content. 
  *     Otherwise a cached result will be used if available
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore
  * @callback: a #GAsyncReadyCallback
  * @user_data: user data passed to @callback
  * 
  * Tries to guess the type of content stored on @mount. Returns one or
  * more textual identifiers of well-known content types (typically
  * prefixed with "x-content/"), e.g. x-content/image-dcf for camera 
- * memory cards. See the 
- * [shared-mime-info](http://www.freedesktop.org/wiki/Specifications/shared-mime-info-spec)
+ * memory cards. See the <ulink url="http://www.freedesktop.org/wiki/Specifications/shared-mime-info-spec">shared-mime-info</ulink>
  * specification for more on x-content types.
  *
  * This is an asynchronous operation (see
@@ -804,13 +776,14 @@ g_mount_guess_content_type (GMount              *mount,
 
   if (iface->guess_content_type == NULL)
     {
-      g_task_report_new_error (mount, callback, user_data,
-                               g_mount_guess_content_type,
-                               G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                               /* Translators: This is an error
-                                * message for mount objects that
-                                * don't implement content type guessing. */
-                               _("mount doesn’t implement content type guessing"));
+      g_simple_async_report_error_in_idle (G_OBJECT (mount),
+                                           callback, user_data,
+                                           G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                                           /* Translators: This is an error
+                                            * message for mount objects that
+                                            * don't implement content type guessing. */
+                                           _("mount doesn't implement content type guessing"));
+
       return;
     }
   
@@ -821,10 +794,10 @@ g_mount_guess_content_type (GMount              *mount,
  * g_mount_guess_content_type_finish:
  * @mount: a #GMount
  * @result: a #GAsyncResult
- * @error: a #GError location to store the error occurring, or %NULL to 
+ * @error: a #GError location to store the error occuring, or %NULL to 
  *     ignore
  * 
- * Finishes guessing content types of @mount. If any errors occurred
+ * Finishes guessing content types of @mount. If any errors occured
  * during the operation, @error will be set to contain the errors and
  * %FALSE will be returned. In particular, you may get an 
  * %G_IO_ERROR_NOT_SUPPORTED if the mount does not support content 
@@ -845,10 +818,12 @@ g_mount_guess_content_type_finish (GMount        *mount,
   g_return_val_if_fail (G_IS_MOUNT (mount), NULL);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
 
-  if (g_async_result_legacy_propagate_error (result, error))
-    return NULL;
-  else if (g_async_result_is_tagged (result, g_mount_guess_content_type))
-    return g_task_propagate_pointer (G_TASK (result), error);
+  if (G_IS_SIMPLE_ASYNC_RESULT (result))
+    {
+      GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+      if (g_simple_async_result_propagate_error (simple, error))
+        return NULL;
+    }
   
   iface = G_MOUNT_GET_IFACE (mount);
   return (* iface->guess_content_type_finish) (mount, result, error);
@@ -859,18 +834,17 @@ g_mount_guess_content_type_finish (GMount        *mount,
  * @mount: a #GMount
  * @force_rescan: Whether to force a rescan of the content.
  *     Otherwise a cached result will be used if available
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore
- * @error: a #GError location to store the error occurring, or %NULL to
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore
+ * @error: a #GError location to store the error occuring, or %NULL to
  *     ignore
  *
  * Tries to guess the type of content stored on @mount. Returns one or
  * more textual identifiers of well-known content types (typically
  * prefixed with "x-content/"), e.g. x-content/image-dcf for camera 
- * memory cards. See the 
- * [shared-mime-info](http://www.freedesktop.org/wiki/Specifications/shared-mime-info-spec)
+ * memory cards. See the <ulink url="http://www.freedesktop.org/wiki/Specifications/shared-mime-info-spec">shared-mime-info</ulink>
  * specification for more on x-content types.
  *
- * This is a synchronous operation and as such may block doing IO;
+ * This is an synchronous operation and as such may block doing IO;
  * see g_mount_guess_content_type() for the asynchronous version.
  *
  * Returns: (transfer full) (element-type utf8): a %NULL-terminated array of content types or %NULL on error.
@@ -897,7 +871,7 @@ g_mount_guess_content_type_sync (GMount              *mount,
                            /* Translators: This is an error
                             * message for mount objects that
                             * don't implement content type guessing. */
-                           _("mount doesn’t implement synchronous content type guessing"));
+                           _("mount doesn't implement synchronous content type guessing"));
 
       return NULL;
     }
@@ -957,10 +931,10 @@ get_private (GMount *mount)
  * situation, a #GVolumeMonitor implementation would create two
  * #GVolume objects (for example, one for the camera functionality of
  * the device and one for a SD card reader on the device) with
- * activation URIs `gphoto2://[usb:001,002]/store1/`
- * and `gphoto2://[usb:001,002]/store2/`. When the
+ * activation URIs <literal>gphoto2://[usb:001,002]/store1/</literal>
+ * and <literal>gphoto2://[usb:001,002]/store2/</literal>. When the
  * underlying mount (with root
- * `gphoto2://[usb:001,002]/`) is mounted, said
+ * <literal>gphoto2://[usb:001,002]/</literal>) is mounted, said
  * #GVolumeMonitor implementation would create two #GMount objects
  * (each with their root matching the corresponding volume activation
  * root) that would shadow the original mount.
@@ -1038,28 +1012,3 @@ g_mount_unshadow (GMount *mount)
     g_warning ("Shadow ref count on GMount is negative");
   G_UNLOCK (priv_lock);
 }
-
-/**
- * g_mount_get_sort_key:
- * @mount: A #GMount.
- *
- * Gets the sort key for @mount, if any.
- *
- * Returns: (nullable): Sorting key for @mount or %NULL if no such key is available.
- *
- * Since: 2.32
- */
-const gchar *
-g_mount_get_sort_key (GMount  *mount)
-{
-  const gchar *ret = NULL;
-  GMountIface *iface;
-
-  g_return_val_if_fail (G_IS_MOUNT (mount), NULL);
-
-  iface = G_MOUNT_GET_IFACE (mount);
-  if (iface->get_sort_key != NULL)
-    ret = iface->get_sort_key (mount);
-
-  return ret;
-}
