@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
@@ -25,63 +27,61 @@
 #include <string.h>
 #include <stdlib.h>
 
-#ifdef G_OS_WIN32
-#include "glib/glib-private.h"
-#endif
-
-static GSettingsSchemaSource   *global_schema_source;
-static GSettings               *global_settings;
-static GSettingsSchema         *global_schema;
-static GSettingsSchemaKey      *global_schema_key;
-const gchar                    *global_key;
-const gchar                    *global_value;
-
 static gboolean
-is_relocatable_schema (GSettingsSchema *schema)
+contained (const gchar * const *items,
+           const gchar         *item)
 {
-  return g_settings_schema_get_path (schema) == NULL;
+  while (*items)
+    if (strcmp (*items++, item) == 0)
+      return TRUE;
+
+  return FALSE;
 }
 
 static gboolean
-check_relocatable_schema (GSettingsSchema *schema,
-                          const gchar     *schema_id)
+is_schema (const gchar *schema)
 {
-  if (schema == NULL)
-    {
-      g_printerr (_("No such schema '%s'\n"), schema_id);
-      return FALSE;
-    }
-
-  if (!is_relocatable_schema (schema))
-    {
-      g_printerr (_("Schema '%s' is not relocatable "
-                    "(path must not be specified)\n"),
-                  schema_id);
-      return FALSE;
-    }
-
-  return TRUE;
+  return contained (g_settings_list_schemas (), schema);
 }
 
 static gboolean
-check_schema (GSettingsSchema *schema,
-              const gchar *schema_id)
+is_relocatable_schema (const gchar *schema)
 {
-  if (schema == NULL)
-    {
-      g_printerr (_("No such schema '%s'\n"), schema_id);
-      return FALSE;
-    }
+  return contained (g_settings_list_relocatable_schemas (), schema);
+}
+
+static gboolean
+check_relocatable_schema (const gchar *schema)
+{
+  if (is_relocatable_schema (schema))
+    return TRUE;
+
+  if (is_schema (schema))
+    g_printerr (_("Schema '%s' is not relocatable "
+                  "(path must not be specified)\n"),
+                schema);
+
+  else
+    g_printerr (_("No such schema '%s'\n"), schema);
+
+  return FALSE;
+}
+
+static gboolean
+check_schema (const gchar *schema)
+{
+  if (is_schema (schema))
+    return TRUE;
 
   if (is_relocatable_schema (schema))
-    {
-      g_printerr (_("Schema '%s' is relocatable "
-                    "(path must be specified)\n"),
-                  schema_id);
-      return FALSE;
-    }
+    g_printerr (_("Schema '%s' is relocatable "
+                  "(path must be specified)\n"),
+                schema);
 
-  return TRUE;
+  else
+    g_printerr (_("No such schema '%s'\n"), schema);
+
+  return FALSE;
 }
 
 static gboolean
@@ -114,8 +114,27 @@ check_path (const gchar *path)
   return TRUE;
 }
 
+static gboolean
+check_key (GSettings   *settings,
+           const gchar *key)
+{
+  gboolean good;
+  gchar **keys;
+
+  keys = g_settings_list_keys (settings);
+  good = contained ((const gchar **) keys, key);
+  g_strfreev (keys);
+
+  if (good)
+    return TRUE;
+
+  g_printerr (_("No such key '%s'\n"), key);
+
+  return FALSE;
+}
+
 static void
-output_list (gchar **list)
+output_list (const gchar * const *list)
 {
   gint i;
 
@@ -124,50 +143,43 @@ output_list (gchar **list)
 }
 
 static void
-gsettings_print_version (void)
+gsettings_list_schemas (GSettings   *settings,
+                        const gchar *key,
+                        const gchar *value)
 {
-  g_print ("%d.%d.%d\n", glib_major_version, glib_minor_version,
-           glib_micro_version);
+  output_list (g_settings_list_schemas ());
 }
 
 static void
-gsettings_list_schemas (void)
+gsettings_list_relocatable_schemas (GSettings   *settings,
+                                    const gchar *key,
+                                    const gchar *value)
 {
-  gchar **schemas;
-
-  g_settings_schema_source_list_schemas (global_schema_source, TRUE, &schemas, NULL);
-  output_list (schemas);
-  g_strfreev (schemas);
+  output_list (g_settings_list_relocatable_schemas ());
 }
 
 static void
-gsettings_list_relocatable_schemas (void)
-{
-  gchar **schemas;
-
-  g_settings_schema_source_list_schemas (global_schema_source, TRUE, NULL, &schemas);
-  output_list (schemas);
-  g_strfreev (schemas);
-}
-
-static void
-gsettings_list_keys (void)
+gsettings_list_keys (GSettings   *settings,
+                     const gchar *key,
+                     const gchar *value)
 {
   gchar **keys;
 
-  keys = g_settings_schema_list_keys (global_schema);
-  output_list (keys);
+  keys = g_settings_list_keys (settings);
+  output_list ((const gchar **) keys);
   g_strfreev (keys);
 }
 
 static void
-gsettings_list_children (void)
+gsettings_list_children (GSettings   *settings,
+                         const gchar *key,
+                         const gchar *value)
 {
   gchar **children;
   gint max = 0;
   gint i;
 
-  children = g_settings_list_children (global_settings);
+  children = g_settings_list_children (settings);
   for (i = 0; children[i]; i++)
     if (strlen (children[i]) > max)
       max = strlen (children[i]);
@@ -175,22 +187,22 @@ gsettings_list_children (void)
   for (i = 0; children[i]; i++)
     {
       GSettings *child;
-      GSettingsSchema *schema;
+      gchar *schema;
       gchar *path;
 
-      child = g_settings_get_child (global_settings, children[i]);
+      child = g_settings_get_child (settings, children[i]);
       g_object_get (child,
-                    "settings-schema", &schema,
+                    "schema", &schema,
                     "path", &path,
                     NULL);
 
-      if (g_settings_schema_get_path (schema) != NULL)
-        g_print ("%-*s   %s\n", max, children[i], g_settings_schema_get_id (schema));
+      if (is_schema (schema))
+        g_print ("%-*s   %s\n", max, children[i], schema);
       else
-        g_print ("%-*s   %s:%s\n", max, children[i], g_settings_schema_get_id (schema), path);
+        g_print ("%-*s   %s:%s\n", max, children[i], schema, path);
 
       g_object_unref (child);
-      g_settings_schema_unref (schema);
+      g_free (schema);
       g_free (path);
     }
 
@@ -201,12 +213,12 @@ static void
 enumerate (GSettings *settings)
 {
   gchar **keys;
-  GSettingsSchema *schema;
+  gchar *schema;
   gint i;
 
-  g_object_get (settings, "settings-schema", &schema, NULL);
+  g_object_get (settings, "schema", &schema, NULL);
 
-  keys = g_settings_schema_list_keys (schema);
+  keys = g_settings_list_keys (settings);
   for (i = 0; keys[i]; i++)
     {
       GVariant *value;
@@ -214,77 +226,54 @@ enumerate (GSettings *settings)
 
       value = g_settings_get_value (settings, keys[i]);
       printed = g_variant_print (value, TRUE);
-      g_print ("%s %s %s\n", g_settings_schema_get_id (schema), keys[i], printed);
+      g_print ("%s %s %s\n", schema, keys[i], printed);
       g_variant_unref (value);
       g_free (printed);
     }
 
-  g_settings_schema_unref (schema);
+  g_free (schema);
   g_strfreev (keys);
 }
 
 static void
-list_recursively (GSettings *settings)
+gsettings_list_recursively (GSettings   *settings,
+                            const gchar *key,
+                            const gchar *value)
 {
   gchar **children;
   gint i;
 
   enumerate (settings);
+
   children = g_settings_list_children (settings);
+
   for (i = 0; children[i]; i++)
     {
       GSettings *child;
+      gchar *schema;
 
       child = g_settings_get_child (settings, children[i]);
-      list_recursively (child);
+      g_object_get (child, "schema", &schema, NULL);
+
+      if (is_schema (schema))
+        enumerate (child);
+
       g_object_unref (child);
+      g_free (schema);
     }
 
   g_strfreev (children);
 }
 
 static void
-gsettings_list_recursively (void)
-{
-  if (global_settings)
-    {
-      list_recursively (global_settings);
-    }
-  else
-    {
-      gchar **schemas;
-      gint i;
-
-      g_settings_schema_source_list_schemas (global_schema_source, TRUE, &schemas, NULL);
-
-      for (i = 0; schemas[i]; i++)
-        {
-          GSettings *settings;
-
-          settings = g_settings_new (schemas[i]);
-          list_recursively (settings);
-          g_object_unref (settings);
-        }
-
-      g_strfreev (schemas);
-    }
-}
-
-static void
-gsettings_description (void)
-{
-  const gchar *description;
-  description = g_settings_schema_key_get_description (global_schema_key);
-  g_print ("%s\n", description);
-}
-
-static void
-gsettings_range (void)
+gsettings_range (GSettings   *settings,
+                 const gchar *key,
+                 const gchar *value)
 {
   GVariant *range, *detail;
   const gchar *type;
 
-  range = g_settings_schema_key_get_range (global_schema_key);
+  range = g_settings_get_range (settings, key);
   g_variant_get (range, "(&sv)", &type, &detail);
 
   if (strcmp (type, "type") == 0)
@@ -330,12 +319,14 @@ gsettings_range (void)
 }
 
 static void
-gsettings_get (void)
+gsettings_get (GSettings   *settings,
+               const gchar *key,
+               const gchar *value_)
 {
   GVariant *value;
   gchar *printed;
 
-  value = g_settings_get_value (global_settings, global_key);
+  value = g_settings_get_value (settings, key);
   printed = g_variant_print (value, TRUE);
   g_print ("%s\n", printed);
   g_variant_unref (value);
@@ -343,62 +334,21 @@ gsettings_get (void)
 }
 
 static void
-gsettings_reset (void)
+gsettings_reset (GSettings   *settings,
+                 const gchar *key,
+                 const gchar *value)
 {
-  g_settings_reset (global_settings, global_key);
+  g_settings_reset (settings, key);
   g_settings_sync ();
 }
 
 static void
-reset_all_keys (GSettings *settings)
-{
-  GSettingsSchema *schema;
-  gchar **keys;
-  gint i;
-
-  g_object_get (settings, "settings-schema", &schema, NULL);
-
-  keys = g_settings_schema_list_keys (schema);
-  for (i = 0; keys[i]; i++)
-    {
-      g_settings_reset (settings, keys[i]);
-    }
-
-  g_settings_schema_unref (schema);
-  g_strfreev (keys);
-}
-
-static void
-gsettings_reset_recursively (void)
-{
-  gchar **children;
-  gint i;
-
-  g_settings_delay (global_settings);
-
-  reset_all_keys (global_settings);
-  children = g_settings_list_children (global_settings);
-  for (i = 0; children[i]; i++)
-    {
-      GSettings *child;
-      child = g_settings_get_child (global_settings, children[i]);
-
-      reset_all_keys (child);
-
-      g_object_unref (child);
-    }
-
-  g_strfreev (children);
-
-  g_settings_apply (global_settings);
-  g_settings_sync ();
-}
-
-static void
-gsettings_writable (void)
+gsettings_writable (GSettings   *settings,
+                    const gchar *key,
+                    const gchar *value)
 {
   g_print ("%s\n",
-           g_settings_is_writable (global_settings, global_key) ?
+           g_settings_is_writable (settings, key) ?
            "true" : "false");
 }
 
@@ -418,33 +368,38 @@ value_changed (GSettings   *settings,
 }
 
 static void
-gsettings_monitor (void)
+gsettings_monitor (GSettings   *settings,
+                   const gchar *key,
+                   const gchar *value)
 {
-  if (global_key)
+  if (key)
     {
       gchar *name;
 
-      name = g_strdup_printf ("changed::%s", global_key);
-      g_signal_connect (global_settings, name, G_CALLBACK (value_changed), NULL);
+      name = g_strdup_printf ("changed::%s", key);
+      g_signal_connect (settings, name, G_CALLBACK (value_changed), NULL);
     }
   else
-    g_signal_connect (global_settings, "changed", G_CALLBACK (value_changed), NULL);
+    g_signal_connect (settings, "changed", G_CALLBACK (value_changed), NULL);
 
-  for (;;)
-    g_main_context_iteration (NULL, TRUE);
+  g_main_loop_run (g_main_loop_new (NULL, FALSE));
 }
 
 static void
-gsettings_set (void)
+gsettings_set (GSettings   *settings,
+               const gchar *key,
+               const gchar *value)
 {
   const GVariantType *type;
   GError *error = NULL;
+  GVariant *existing;
   GVariant *new;
   gchar *freeme = NULL;
 
-  type = g_settings_schema_key_get_value_type (global_schema_key);
+  existing = g_settings_get_value (settings, key);
+  type = g_variant_get_type (existing);
 
-  new = g_variant_parse (type, global_value, NULL, NULL, &error);
+  new = g_variant_parse (type, value, NULL, NULL, &error);
 
   /* If that didn't work and the type is string then we should assume
    * that the user is just trying to set a string directly and forgot
@@ -469,33 +424,28 @@ gsettings_set (void)
    */
   if (new == NULL &&
       g_variant_type_equal (type, G_VARIANT_TYPE_STRING) &&
-      global_value[0] != '\'' && global_value[0] != '"')
+      value[0] != '\'' && value[0] != '"')
     {
       g_clear_error (&error);
-      new = g_variant_new_string (global_value);
+      new = g_variant_new_string (value);
     }
 
   if (new == NULL)
     {
-      gchar *context;
-
-      context = g_variant_parse_error_print_context (error, global_value);
-      g_printerr ("%s", context);
+      g_printerr ("%s\n", error->message);
       exit (1);
     }
 
-  if (!g_settings_schema_key_range_check (global_schema_key, new))
+  if (!g_settings_range_check (settings, key, new))
     {
       g_printerr (_("The provided value is outside of the valid range\n"));
       g_variant_unref (new);
       exit (1);
     }
 
-  if (!g_settings_set_value (global_settings, global_key, new))
-    {
-      g_printerr (_("The key is not writable\n"));
-      exit (1);
-    }
+  g_settings_set_value (settings, key, new);
+  g_variant_unref (existing);
+  g_variant_unref (new);
 
   g_settings_sync ();
 
@@ -521,12 +471,6 @@ gsettings_help (gboolean     requested,
       synopsis = "[COMMAND]";
     }
 
-  else if (strcmp (command, "--version") == 0)
-    {
-      description = _("Print version information and exit");
-      synopsis = "";
-    }
-
   else if (strcmp (command, "list-schemas") == 0)
     {
       description = _("List the installed (non-relocatable) schemas");
@@ -553,9 +497,8 @@ gsettings_help (gboolean     requested,
 
   else if (strcmp (command, "list-recursively") == 0)
     {
-      description = _("List keys and values, recursively\n"
-                      "If no SCHEMA is given, list all keys\n");
-      synopsis = N_("[SCHEMA[:PATH]]");
+      description = _("List keys and values, recursively");
+      synopsis = N_("SCHEMA[:PATH]");
     }
 
   else if (strcmp (command, "get") == 0)
@@ -570,12 +513,6 @@ gsettings_help (gboolean     requested,
       synopsis = N_("SCHEMA[:PATH] KEY");
     }
 
-  else if (strcmp (command, "describe") == 0)
-    {
-      description = _("Query the description for KEY");
-      synopsis = N_("SCHEMA[:PATH] KEY");
-    }
-
   else if (strcmp (command, "set") == 0)
     {
       description = _("Set the value of KEY to VALUE");
@@ -588,12 +525,6 @@ gsettings_help (gboolean     requested,
       synopsis = N_("SCHEMA[:PATH] KEY");
     }
 
-  else if (strcmp (command, "reset-recursively") == 0)
-    {
-      description = _("Reset all keys in SCHEMA to their defaults");
-      synopsis = N_("SCHEMA[:PATH]");
-    }
-
   else if (strcmp (command, "writable") == 0)
     {
       description = _("Check if KEY is writable");
@@ -618,8 +549,7 @@ gsettings_help (gboolean     requested,
     {
       g_string_append (string,
       _("Usage:\n"
-        "  gsettings --version\n"
-        "  gsettings [--schemadir SCHEMADIR] COMMAND [ARGS...]\n"
+        "  gsettings COMMAND [ARGS...]\n"
         "\n"
         "Commands:\n"
         "  help                      Show this information\n"
@@ -629,11 +559,9 @@ gsettings_help (gboolean     requested,
         "  list-children             List children of a schema\n"
         "  list-recursively          List keys and values, recursively\n"
         "  range                     Queries the range of a key\n"
-        "  describe                  Queries the description of a key\n"
         "  get                       Get the value of a key\n"
         "  set                       Set the value of a key\n"
         "  reset                     Reset the value of a key\n"
-        "  reset-recursively         Reset all values in a given schema\n"
         "  writable                  Check if a key is writable\n"
         "  monitor                   Watch for changes\n"
         "\n"
@@ -641,42 +569,42 @@ gsettings_help (gboolean     requested,
     }
   else
     {
-      g_string_append_printf (string, _("Usage:\n  gsettings [--schemadir SCHEMADIR] %s %s\n\n%s\n\n"),
-                              command, synopsis[0] ? _(synopsis) : "", description);
+      g_string_append_printf (string, _("Usage:\n  gsettings %s %s\n\n%s\n\n"),
+                              command, _(synopsis), description);
 
-      g_string_append (string, _("Arguments:\n"));
+      if (synopsis[0])
+        {
+          g_string_append (string, _("Arguments:\n"));
 
-      g_string_append (string,
-                       _("  SCHEMADIR A directory to search for additional schemas\n"));
+          if (strstr (synopsis, "[COMMAND]"))
+            g_string_append (string,
+                           _("  COMMAND   The (optional) command to explain\n"));
 
-      if (strstr (synopsis, "[COMMAND]"))
-        g_string_append (string,
-                       _("  COMMAND   The (optional) command to explain\n"));
+          else if (strstr (synopsis, "SCHEMA"))
+            g_string_append (string,
+                           _("  SCHEMA    The name of the schema\n"
+                             "  PATH      The path, for relocatable schemas\n"));
 
-      else if (strstr (synopsis, "SCHEMA"))
-        g_string_append (string,
-                       _("  SCHEMA    The name of the schema\n"
-                         "  PATH      The path, for relocatable schemas\n"));
+          if (strstr (synopsis, "[KEY]"))
+            g_string_append (string,
+                           _("  KEY       The (optional) key within the schema\n"));
 
-      if (strstr (synopsis, "[KEY]"))
-        g_string_append (string,
-                       _("  KEY       The (optional) key within the schema\n"));
+          else if (strstr (synopsis, "KEY"))
+            g_string_append (string,
+                           _("  KEY       The key within the schema\n"));
 
-      else if (strstr (synopsis, "KEY"))
-        g_string_append (string,
-                       _("  KEY       The key within the schema\n"));
+          if (strstr (synopsis, "VALUE"))
+            g_string_append (string,
+                           _("  VALUE     The value to set\n"));
 
-      if (strstr (synopsis, "VALUE"))
-        g_string_append (string,
-                       _("  VALUE     The value to set\n"));
-
-      g_string_append (string, "\n");
+          g_string_append (string, "\n");
+        }
     }
 
   if (requested)
     g_print ("%s", string->str);
   else
-    g_printerr ("%s\n", string->str);
+    g_printerr ("%s", string->str);
 
   g_string_free (string, TRUE);
 
@@ -687,68 +615,18 @@ gsettings_help (gboolean     requested,
 int
 main (int argc, char **argv)
 {
-  void (* function) (void);
-  gboolean need_settings;
-
-#ifdef G_OS_WIN32
-  gchar *tmp;
-#endif
+  void (* function) (GSettings *, const gchar *, const gchar *);
+  GSettings *settings;
+  const gchar *key;
 
   setlocale (LC_ALL, "");
-  textdomain (GETTEXT_PACKAGE);
-
-#ifdef G_OS_WIN32
-  tmp = _glib_get_locale_dir ();
-  bindtextdomain (GETTEXT_PACKAGE, tmp);
-  g_free (tmp);
-#else
-  bindtextdomain (GETTEXT_PACKAGE, GLIB_LOCALE_DIR);
-#endif
-
-#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#endif
 
   if (argc < 2)
     return gsettings_help (FALSE, NULL);
 
-  global_schema_source = g_settings_schema_source_get_default ();
-
-  if (argc > 3 && g_str_equal (argv[1], "--schemadir"))
-    {
-      GSettingsSchemaSource *parent = global_schema_source;
-      GError *error = NULL;
-
-      global_schema_source = g_settings_schema_source_new_from_directory (argv[2], parent, FALSE, &error);
-
-      if (global_schema_source == NULL)
-        {
-          g_printerr (_("Could not load schemas from %s: %s\n"), argv[2], error->message);
-          g_clear_error (&error);
-
-          return 1;
-        }
-
-      /* shift remaining arguments (not correct wrt argv[0], but doesn't matter) */
-      argv = argv + 2;
-      argc -= 2;
-    }
-  else if (global_schema_source == NULL)
-    {
-      g_printerr (_("No schemas installed\n"));
-      return 1;
-    }
-  else
-    g_settings_schema_source_ref (global_schema_source);
-
-  need_settings = TRUE;
-
-  if (strcmp (argv[1], "help") == 0)
+  else if (strcmp (argv[1], "help") == 0)
     return gsettings_help (TRUE, argv[2]);
 
-  else if (argc == 2 && strcmp (argv[1], "--version") == 0)
-    function = gsettings_print_version;
-
   else if (argc == 2 && strcmp (argv[1], "list-schemas") == 0)
     function = gsettings_list_schemas;
 
@@ -756,28 +634,16 @@ main (int argc, char **argv)
     function = gsettings_list_relocatable_schemas;
 
   else if (argc == 3 && strcmp (argv[1], "list-keys") == 0)
-    {
-      need_settings = FALSE;
-      function = gsettings_list_keys;
-    }
+    function = gsettings_list_keys;
 
   else if (argc == 3 && strcmp (argv[1], "list-children") == 0)
     function = gsettings_list_children;
 
-  else if ((argc == 2 || argc == 3) && strcmp (argv[1], "list-recursively") == 0)
+  else if (argc == 3 && strcmp (argv[1], "list-recursively") == 0)
     function = gsettings_list_recursively;
 
-  else if (argc == 4 && strcmp (argv[1], "describe") == 0)
-    {
-      need_settings = FALSE;
-      function = gsettings_description;
-    }
-
   else if (argc == 4 && strcmp (argv[1], "range") == 0)
-    {
-      need_settings = FALSE;
-      function = gsettings_range;
-    }
+    function = gsettings_range;
 
   else if (argc == 4 && strcmp (argv[1], "get") == 0)
     function = gsettings_get;
@@ -788,9 +654,6 @@ main (int argc, char **argv)
   else if (argc == 4 && strcmp (argv[1], "reset") == 0)
     function = gsettings_reset;
 
-  else if (argc == 3 && strcmp (argv[1], "reset-recursively") == 0)
-    function = gsettings_reset_recursively;
-
   else if (argc == 4 && strcmp (argv[1], "writable") == 0)
     function = gsettings_writable;
 
@@ -800,85 +663,54 @@ main (int argc, char **argv)
   else
     return gsettings_help (FALSE, argv[1]);
 
+  g_type_init ();
+
   if (argc > 2)
     {
       gchar **parts;
 
       if (argv[2][0] == '\0')
         {
-          g_printerr (_("Empty schema name given\n"));
+          g_printerr (_("Empty schema name given"));
           return 1;
         }
 
       parts = g_strsplit (argv[2], ":", 2);
 
-      global_schema = g_settings_schema_source_lookup (global_schema_source, parts[0], TRUE);
-
-      if (need_settings)
+      if (parts[1])
         {
-          if (parts[1])
-            {
-              if (!check_relocatable_schema (global_schema, parts[0]) || !check_path (parts[1]))
-                return 1;
+          if (!check_relocatable_schema (parts[0]) || !check_path (parts[1]))
+            return 1;
 
-              global_settings = g_settings_new_full (global_schema, NULL, parts[1]);
-            }
-          else
-            {
-              if (!check_schema (global_schema, parts[0]))
-                return 1;
-
-              global_settings = g_settings_new_full (global_schema, NULL, NULL);
-            }
+          settings = g_settings_new_with_path (parts[0], parts[1]);
         }
       else
         {
-          /* If the user has given a path then we enforce that we have a
-           * relocatable schema, but if they didn't give a path then it
-           * doesn't matter what type of schema we have (since it's
-           * reasonable to ask for introspection information on a
-           * relocatable schema without having to give the path).
-           */
-          if (parts[1])
-            {
-              if (!check_relocatable_schema (global_schema, parts[0]) || !check_path (parts[1]))
-                return 1;
-            }
-          else
-            {
-              if (global_schema == NULL)
-                {
-                  g_printerr (_("No such schema '%s'\n"), parts[0]);
-                  return 1;
-                }
-            }
+          if (!check_schema (parts[0]))
+            return 1;
+
+          settings = g_settings_new (parts[0]);
         }
 
       g_strfreev (parts);
     }
+  else
+    settings = NULL;
 
   if (argc > 3)
     {
-      if (!g_settings_schema_has_key (global_schema, argv[3]))
-        {
-          g_printerr (_("No such key '%s'\n"), argv[3]);
-          return 1;
-        }
+      if (!check_key (settings, argv[3]))
+        return 1;
 
-      global_key = argv[3];
-      global_schema_key = g_settings_schema_get_key (global_schema, global_key);
+      key = argv[3];
     }
+  else
+    key = NULL;
 
-  if (argc > 4)
-    global_value = argv[4];
+  (* function) (settings, key, argc > 4 ? argv[4] : NULL);
 
-  (* function) ();
-
-
-  g_clear_pointer (&global_schema_source, g_settings_schema_source_unref);
-  g_clear_pointer (&global_schema_key, g_settings_schema_key_unref);
-  g_clear_pointer (&global_schema, g_settings_schema_unref);
-  g_clear_object (&global_settings);
+  if (settings != NULL)
+    g_object_unref (settings);
 
   return 0;
 }
