@@ -1,13 +1,11 @@
 /* GIO - GLib Input, Output and Streaming Library
- *
+ * 
  * Copyright (C) 2008 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,36 +13,19 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
 #include "gsocketaddressenumerator.h"
 #include "glibintl.h"
 
-#include "gtask.h"
+#include "gsimpleasyncresult.h"
 
-/**
- * GSocketAddressEnumerator:
- *
- * `GSocketAddressEnumerator` is an enumerator type for
- * [class@Gio.SocketAddress] instances. It is returned by enumeration functions
- * such as [method@Gio.SocketConnectable.enumerate], which returns a
- * `GSocketAddressEnumerator` to list each [class@Gio.SocketAddress] which could
- * be used to connect to that [iface@Gio.SocketConnectable].
- *
- * Enumeration is typically a blocking operation, so the asynchronous methods
- * [method@Gio.SocketAddressEnumerator.next_async] and
- * [method@Gio.SocketAddressEnumerator.next_finish] should be used where
- * possible.
- *
- * Each `GSocketAddressEnumerator` can only be enumerated once. Once
- * [method@Gio.SocketAddressEnumerator.next] has returned `NULL`, further
- * enumeration with that `GSocketAddressEnumerator` is not possible, and it can
- * be unreffed.
- */
 
-G_DEFINE_ABSTRACT_TYPE (GSocketAddressEnumerator, g_socket_address_enumerator, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE (GSocketAddressEnumerator, g_socket_address_enumerator, G_TYPE_OBJECT);
 
 static void            g_socket_address_enumerator_real_next_async  (GSocketAddressEnumerator  *enumerator,
 								     GCancellable              *cancellable,
@@ -69,7 +50,7 @@ g_socket_address_enumerator_class_init (GSocketAddressEnumeratorClass *enumerato
 /**
  * g_socket_address_enumerator_next:
  * @enumerator: a #GSocketAddressEnumerator
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
  * @error: a #GError.
  *
  * Retrieves the next #GSocketAddress from @enumerator. Note that this
@@ -81,13 +62,13 @@ g_socket_address_enumerator_class_init (GSocketAddressEnumeratorClass *enumerato
  * If @enumerator is expected to yield addresses, but for some reason
  * is unable to (eg, because of a DNS error), then the first call to
  * g_socket_address_enumerator_next() will return an appropriate error
- * in `*error`. However, if the first call to
+ * in *@error. However, if the first call to
  * g_socket_address_enumerator_next() succeeds, then any further
  * internal errors (other than @cancellable being triggered) will be
  * ignored.
  *
- * Returns: (transfer full) (nullable): a #GSocketAddress (owned by the caller), or %NULL on
- *     error (in which case `*error` will be set) or if there are no
+ * Return value: (transfer full): a #GSocketAddress (owned by the caller), or %NULL on
+ *     error (in which case *@error will be set) or if there are no
  *     more addresses.
  */
 GSocketAddress *
@@ -114,35 +95,34 @@ g_socket_address_enumerator_real_next_async (GSocketAddressEnumerator *enumerato
 					     GAsyncReadyCallback       callback,
 					     gpointer                  user_data)
 {
-  GTask *task;
+  GSimpleAsyncResult *result;
   GSocketAddress *address;
   GError *error = NULL;
 
-  task = g_task_new (enumerator, NULL, callback, user_data);
-  g_task_set_source_tag (task, g_socket_address_enumerator_real_next_async);
-
+  result = g_simple_async_result_new (G_OBJECT (enumerator),
+				      callback, user_data,
+				      g_socket_address_enumerator_real_next_async);
   address = g_socket_address_enumerator_next (enumerator, cancellable, &error);
-  if (error)
-    g_task_return_error (task, error);
-  else
-    g_task_return_pointer (task, address, g_object_unref);
+  if (address)
+    g_simple_async_result_set_op_res_gpointer (result, address, NULL);
+  else if (error)
+    g_simple_async_result_take_error (result, error);
 
-  g_object_unref (task);
+  g_simple_async_result_complete_in_idle (result);
+  g_object_unref (result);
 }
 
 /**
  * g_socket_address_enumerator_next_async:
  * @enumerator: a #GSocketAddressEnumerator
- * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
- * @callback: (scope async) (closure user_data): a #GAsyncReadyCallback to call
- *   when the request is satisfied
- * @user_data: the data to pass to callback function
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
+ * @callback: (scope async): a #GAsyncReadyCallback to call when the request
+ *     is satisfied
+ * @user_data: (closure): the data to pass to callback function
  *
  * Asynchronously retrieves the next #GSocketAddress from @enumerator
  * and then calls @callback, which must call
  * g_socket_address_enumerator_next_finish() to get the result.
- *
- * It is an error to call this multiple times before the previous callback has finished.
  */
 void
 g_socket_address_enumerator_next_async (GSocketAddressEnumerator *enumerator,
@@ -164,9 +144,16 @@ g_socket_address_enumerator_real_next_finish (GSocketAddressEnumerator  *enumera
 					      GAsyncResult              *result,
 					      GError                   **error)
 {
-  g_return_val_if_fail (g_task_is_valid (result, enumerator), NULL);
+  GSimpleAsyncResult *simple;
 
-  return g_task_propagate_pointer (G_TASK (result), error);
+  g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (result), NULL);
+  simple = G_SIMPLE_ASYNC_RESULT (result);
+  g_return_val_if_fail (g_simple_async_result_get_source_tag (simple) == g_socket_address_enumerator_real_next_async, NULL);
+
+  if (g_simple_async_result_propagate_error (simple, error))
+    return NULL;
+  else
+    return g_simple_async_result_get_op_res_gpointer (simple);
 }
 
 /**
@@ -180,8 +167,8 @@ g_socket_address_enumerator_real_next_finish (GSocketAddressEnumerator  *enumera
  * g_socket_address_enumerator_next() for more information about
  * error handling.
  *
- * Returns: (transfer full) (nullable): a #GSocketAddress (owned by the caller), or %NULL on
- *     error (in which case `*error` will be set) or if there are no
+ * Return value: (transfer full): a #GSocketAddress (owned by the caller), or %NULL on
+ *     error (in which case *@error will be set) or if there are no
  *     more addresses.
  */
 GSocketAddress *
